@@ -19,12 +19,13 @@
 static char *usage[] = {"Perseus - slays monsters\n",
 			"-sin     string            seq file name\n",
 			"Options:\n",
+			"-s       integer\n",
 			"-tin     string            reference sequence file\n",
 			"-a                         output alignments\n",
 			"-d                         use imbalance\n",
 			"-rin     string            lookup file name\n"};
 
-static int  nLines = 7;
+static int  nLines = 8;
 
 static char szSequence[] = "ACGTNacgtn-";
 
@@ -44,15 +45,26 @@ int main(int argc, char* argv[]){
   /*get command line params*/
   getCommandLineParams(&tParams, argc, argv);
   
+  /*read sequences to chimera check*/
   readData(tParams.szSeqInputFile, &tSeqData);
 
+  /*read sequences to compare too*/
   readData(tParams.szRefInputFile, &tRefData);
  
+  /*set parameters for sequence distances*/
   initLookUp(&tParams);
 
-  /*do my bit*/
-  nN = tSeqData.nSeq; nK = tRefData.nSeq;
-  nNLen = tSeqData.nMaxLen; nKLen = tRefData.nMaxLen;
+  /*number of sequences*/
+  nN = tSeqData.nSeq; 
+
+  /*number of reference sequences*/
+  nK = tRefData.nSeq;
+  
+  /*max length of sequences*/
+  nNLen = tSeqData.nMaxLen; 
+
+  /*max length of references*/
+  nKLen = tRefData.nMaxLen;
 
   printf("%d %d\n",nN, nK);
 
@@ -70,37 +82,13 @@ int main(int argc, char* argv[]){
 
     printf("%d %s ",i, tSeqData.aszID[i]);
 
-    /*do pairwise alignments and get best hit*/
-    nCompare = alignAll(i, nLenI, &nBest, &nBestJ, anRestrict, nK, &tSeqData, &tRefData, atAlign);
+    /*do pairwise alignments and get best hit for each sequence i*/
+    nCompare = alignAll(i, nLenI, &nBest, &nBestJ, anRestrict, nK, &tSeqData, &tRefData, atAlign, &tParams);
 
     if(nCompare >= 2){
 
       nBestChi = getBestChimera(nK, &tRefData, &nP1, &nP2, &nSplit, anRestrict, nLenI, atAlign, anD, anR, anBestD, anBestR);
-      if(0)
-      {
-	int a1 = 0, a2 = 0;
 
-	for(a = 0; a < nLenI; a++){
-	  printf("%d %c ", a, tSeqData.acSequences[nNLen*i + a]);
-
-	  while((atAlign[nP1].acA[a1] == GAP || atAlign[nP1].acA[a1] == T_GAP) && a1 < atAlign[nP1].nLen){
-	    a1++;
-	  }
-
-	  printf("%c %d %d ", atAlign[nP1].acB[a1], atAlign[nP1].anD[a], atAlign[nP1].anR[nLenI - a - 2]);
-	  a1++;
-
-	  while((atAlign[nP2].acA[a2] == GAP || atAlign[nP2].acA[a2] == T_GAP) && a2 < atAlign[nP2].nLen ){
-	    a2++;
-	  }
-
-	  printf("%c %d %d ", atAlign[nP2].acB[a2], atAlign[nP2].anD[a], atAlign[nP2].anR[nLenI - a - 2]);
-	  a2++;
-
-	  printf("\n");
-	}
-	
-      }
       if(nBestChi >= 3 && nCompare >= 3){
 	nBestTri = getBestTrimera(nK, &tRefData, &nT1, &nT2, &nT3, &nSplit1, &nSplit2, anRestrict, nLenI, atAlign, anD, anR, anBestD, anBestR);
       }
@@ -235,6 +223,17 @@ void getCommandLineParams(t_Params *ptParams,int argc,char *argv[])
     ptParams->szRefInputFile = ptParams->szSeqInputFile;
   }
 
+  szTemp  = extractParameter(argc,argv,SKEW,OPTION);  
+  if(szTemp != NULL){
+    ptParams->nSkew = strtol(szTemp,&cError,10);
+    if(*cError!='\0'){
+      goto error;
+    }
+  }
+  else{
+    ptParams->nSkew = DEFAULT_SKEW;
+  }
+
   if(extractParameter(argc,argv,USE_IMBALANCE,OPTION)){  
     ptParams->bImbalance = TRUE;
   }
@@ -1484,8 +1483,9 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
   char* acSequences = NULL;
   double dRet = 0;
   char szTempFasta[MAX_LINE_LENGTH], szTempAlign[MAX_LINE_LENGTH];
-  int nO1 = 0, nO2 = 0, nO3 = 0, nOffset = 0;
+  int nTGap1 = 0, nTGap2 = 0, nTGap3 = 0,nMaxTGap = -1;
 
+  /*create sequence filenames*/
   if(ptParams->bOutputAlignments == FALSE){
   	sprintf(szTempFasta, "Temp%s",FASTA_SUFFIX);
   	sprintf(szTempAlign, "Temp%s",ALIGN_SUFFIX);
@@ -1495,8 +1495,8 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
   	sprintf(szTempAlign, "Temp%d%s",nI,ALIGN_SUFFIX);
   }
 
+  /*write sequences*/
   ofp = fopen(szTempFasta, "w");
-
   if(ofp){
     writeSequenceI(ofp, ptSeqData, nI);
     writeSequenceI(ofp, ptRefData, nP1);
@@ -1507,45 +1507,38 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
     fprintf(stderr, "Failed to open %s for writing ... abort\n",szTempFasta);
     exit(EXIT_FAILURE);
   }
-  
+  /*run mafft remotely*/
   sprintf(szCommand,"mafft-linsi %s > %s 2> %s",szTempFasta,szTempAlign, TEMP_ERROR_FILE);
 
   system(szCommand);
 
+  /*read in mafft output - three sequence alignment*/
   readData(szTempAlign, &tAlign);
  
   acSequences = tAlign.acSequences;  nMaxLen = tAlign.nMaxLen;
-  //nO1 = calcOffset(acSequences, nMaxLen); nO2 = calcOffset(acSequences + nMaxLen, nMaxLen); nO3 = calcOffset(acSequences + 2*nMaxLen, nMaxLen);
 
-  //nOffset =  nO1 > nO2 ? nO1 : nO2;
-  //nOffset = nO3 > nOffset ? nO3 : nOffset;
- 
+  /*alignment length*/
   nTLen = tAlign.nMaxLen;
 
-  if(1){
-	int nTGap1 = 0, nTGap2 = 0, nTGap3 = 0,nMaxTGap = -1;
-
-  	while(acSequences[nTLen - 1 - nTGap1] == GAP && nTLen - nTGap1> 1){
-    		nTGap1++;
-  	}
-
-  	while(acSequences[nMaxLen + nTLen - 1 - nTGap2] == GAP && nTLen - nTGap2> 1){
-    		nTGap2++;
-  	}
-
-  	while(acSequences[2*nMaxLen + nTLen - 1 - nTGap3] == GAP && nTLen - nTGap3> 1){
-    		nTGap3++;
-  	}
-
-	nMaxTGap = nTGap1 > nTGap2 ? nTGap1 : nTGap2;		
-	nMaxTGap = nTGap3 > nMaxTGap ? nTGap3 : nMaxTGap;
-	nTLen -= nMaxTGap;
-	//printf("%d %d %d %d\n",nTGap1,nTGap2,nTGap3, nMaxTGap);
-  }	
+  /*find largest terminal gap*/
+  while(acSequences[nTLen - 1 - nTGap1] == GAP && nTLen - nTGap1> 1){
+    nTGap1++;
+  }
 
+  while(acSequences[nMaxLen + nTLen - 1 - nTGap2] == GAP && nTLen - nTGap2> 1){
+    nTGap2++;
+  }
 
+  while(acSequences[2*nMaxLen + nTLen - 1 - nTGap3] == GAP && nTLen - nTGap3> 1){
+    nTGap3++;
+  }
 
+  nMaxTGap = nTGap1 > nTGap2 ? nTGap1 : nTGap2;		
+  nMaxTGap = nTGap3 > nMaxTGap ? nTGap3 : nMaxTGap;
+  /*remove from alignment*/
+  nTLen -= nMaxTGap;	
 
+  /*find all differences between chimera and parents and positions*/
   for(i = 0; i < nTLen; i++){
     if(acSequences[i] != acSequences[nMaxLen + i]){
       if(nDiff1 < MAX_DIFF){
@@ -1576,7 +1569,7 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
   anSplits[s] = -1;
   anD[s] = nDiff2;
   s++;
-
+  /*loop differences to find optimal split point*/
   while(s1 < nDiff1 || s2 < nDiff2){
     if(anDiff1[s1] <= anDiff2[s2]){
       anD[s] = anD[s - 1] + 1;
@@ -1602,6 +1595,7 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
       sMin = i;
     }
   }
+  
   if(sMin < s - 1){
     nSplit = (anSplits[sMin] + anSplits[sMin + 1])/2;
   }
@@ -1609,6 +1603,7 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
     nSplit = nTLen - 1;
   }
 
+  /*dummy if, as we always do this*/
   if(TRUE){
     int nA = -1, nB = -1;
     int nDLP1 = 0, nDRP1 = 0, nDLP2 = 0, nDRP2 = 0;
@@ -1620,22 +1615,25 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
 
     for(i = 0; i < nTLen; i++){
       char cI = acSequences[i], cP1 = acSequences[nMaxLen + i], cP2 = acSequences[2*nMaxLen + i];
-
+      /*get consenus base*/
       cC = getC(cI, cP1, cP2);
 
+      /*count number of differences between chimera and consensus*/
       if(cI != cC){
 	nZ++;
       }
-
+      /*count number of differences between chimera and parent 1*/
       if(cP1 != cC){
+	/*count number to left*/
 	if(i <= nSplit){
 	  nDLP1++;
 	}
+	/*count number to right*/
 	else{
 	  nDRP1++;
 	}
       }
-      
+      /*count number of differences between chimera and parent 2*/
       if(cP2 != cC){
 	if(i <= nSplit){
 	  nDLP2++;
@@ -1645,7 +1643,7 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
 	}
       }
     }
-
+    /*if Parent1 is left*/
     if(nDiff1 <= nDiff2){
       nA = nP1;
       nB = nP2;
@@ -1656,10 +1654,11 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
       nY = nDLP2 + nDRP2;
       nYA = nDLP2; nYB = nDRP2;
 
-      pA = ((double) nSplit + 1)/((double) nTLen);
-      pB = ((double) nTLen - nSplit - 1)/((double) nTLen);;
+      pA = ((double) nSplit + 1)/((double) nTLen); /*probability of change to left*/
+      pB = ((double) nTLen - nSplit - 1)/((double) nTLen);/*prob. to right*/
     }
     else{
+      /*if parent 1 is right*/
       nA = nP2;
       nB = nP1;
 
@@ -1670,7 +1669,7 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
       nYA = nDRP1; nYB = nDLP1;
 
       pB = ((double) nSplit + 1)/((double) nTLen);
-      pA = ((double) nTLen - nSplit - 1)/((double) nTLen);;
+      pA = ((double) nTLen - nSplit - 1)/((double) nTLen);
     }
 
     nXZ = nX + nZ;
@@ -1678,6 +1677,7 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
     dRet = 0.0;
     dP = 0.0;
     
+    /*adds extra factor for tree imbalance - not generally used*/
     if(ptParams->bImbalance){
       if(nXZ > 0){
 	for(i = nX; i <= nXZ; i++){
@@ -1688,6 +1688,7 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
       }
     }
 
+    /*contribution from right parent*/
     if(nY > 0){
       dP = 0.0;
       for(i = nYA; i <= nY; i++){
@@ -1696,6 +1697,7 @@ double calcLoonIndex(t_Data *ptSeqData, t_Data *ptRefData, int nI, int nP1, int
       dRet += -log(dP);
     }
 
+    /*contribution from left*/
     if(nX > 0){
       dP = 0.0;
       for(i = nXB; i <= nX; i++){
@@ -1727,7 +1729,7 @@ void destroyData(t_Data *ptData)
   }
 }
 
-int alignAll(int nI, int nLenI, int *pnBest, int *pnBestJ, int *anRestrict, int nK, t_Data *ptSeqData, t_Data *ptRefData, t_Align* atAlign)
+int alignAll(int nI, int nLenI, int *pnBest, int *pnBestJ, int *anRestrict, int nK, t_Data *ptSeqData, t_Data *ptRefData, t_Align* atAlign, t_Params *ptParams)
 {
   int j = 0, nCompare = 0, nBest = BIG_INT, nBestJ = -1;
   int nNLen = ptSeqData->nMaxLen, nKLen = ptRefData->nMaxLen;
@@ -1735,7 +1737,7 @@ int alignAll(int nI, int nLenI, int *pnBest, int *pnBestJ, int *anRestrict, int
   for(j = 0; j < nK; j++){
     int     nDist = 0;
 
-    if(strcmp(ptRefData->aszID[j],ptSeqData->aszID[nI]) != 0 && ptRefData->adFreq[j] >= ptSeqData->adFreq[nI]){
+    if(strcmp(ptRefData->aszID[j],ptSeqData->aszID[nI]) != 0 && ptRefData->adFreq[j] >= ptParams->nSkew*ptSeqData->adFreq[nI]){
 
       needlemanWunsch(&atAlign[j], &ptSeqData->acSequences[nI*nNLen], &ptRefData->acSequences[j*nKLen], nLenI, ptRefData->anLen[j]);
       
@@ -1873,3 +1875,30 @@ int getBestTrimera(int nK, t_Data *ptRefData, int* pnT1, int* pnT2, int* pnT3, i
 
   return nBestTri;
 }
+
+/*void writeAlignment(int nLenI)
+{
+  int a = 0, a1 = 0, a2 = 0;
+
+  for(a = 0; a < nLenI; a++){
+    printf("%d %c ", a, tSeqData.acSequences[nNLen*i + a]);
+
+    while((atAlign[nP1].acA[a1] == GAP || atAlign[nP1].acA[a1] == T_GAP) && a1 < atAlign[nP1].nLen){
+      a1++;
+    }
+
+    printf("%c %d %d ", atAlign[nP1].acB[a1], atAlign[nP1].anD[a], atAlign[nP1].anR[nLenI - a - 2]);
+    a1++;
+
+    while((atAlign[nP2].acA[a2] == GAP || atAlign[nP2].acA[a2] == T_GAP) && a2 < atAlign[nP2].nLen ){
+	    a2++;
+    }
+
+    printf("%c %d %d ", atAlign[nP2].acB[a2], atAlign[nP2].anD[a], atAlign[nP2].anR[nLenI - a - 2]);
+    a2++;
+
+    printf("\n");
+  }
+	
+ }
+ }*/
